#include "cliargs.h"
#include "dyn_arr.h"
#include <bits/getopt_core.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

void print_help(char*);
void print_version(char*);

DEFINE_DA_IMPL(string, char*);

cliargs_t* parse_cliargs(int argc, char** argv) {
    int optidx = 0;
    int opt;
    char* progname = argv[0];
    
    if (argc <= 1) {
        print_help(progname);
    }

    cliargs_t* cliargs = malloc(sizeof(cliargs_t));
    assert(cliargs && "Out of memory");
    
    while ((opt = getopt_long(argc, argv, "hvo:d", cli_options, &optidx)) != -1) {
        switch (opt) {
            case 'h':
                print_help(progname);
                break;
            case 'v':
                print_version(progname);
                break;
            case 'o':
                cliargs->output = optarg;
                break;
            case 'd':
                cliargs->debug = true;
                break;
            default: /* '?' */
                print_help(progname);
                exit(EXIT_FAILURE);

        }
    }

    for (int i = optind; i < argc; i++) {
        mcl_da_string_push(&cliargs->input, argv[i]);
    }

    return cliargs;
}

void print_help(char* progname) {
    printf(
        "Usage: %s [-v|--version] [-h|--help] [-d|--debug] [-o|--output OUTPUT] [INPUT]\n"
        "    --version, -v    -   Prints the current version\n"
        "    --help,    -h    -   Shows this text\n"
        "    --debug,   -d    -   Prints extra information\n"
        "    --output,  -o    -   Output file for compilation\n"
        "\n"
        "%s Copyright (C) 2024  MCorange<mcorange@mcorangehq.xyz>\n"
        "This program comes with ABSOLUTELY NO WARRANTY.\n"
        "This is free software, and you are welcome to redistribute it\n"
        "under certain conditions\n"
    , progname, progname);
    exit(0);
}

void print_version(char* progname) {
   printf("%s: The very epic compiler %s\n", progname, MCL_VERSION); 
   exit(0);
}
